#ifndef AMDIS_TRANSLATELSFCT_H
#define AMDIS_TRANSLATELSFCT_H

#include "ElementFunction.h"
#include "FixVec.h"

using namespace AMDiS;

template<typename T>
class TranslateLsFct : public ElementFunction<T>
{
 public:
  /**
   * Constructor
   */
  TranslateLsFct(ElementFunction<T> *f_,
		 double c_)
    : ElementFunction<T>(),
    f(f_),
    c(c_)
  {};

  /** 
   * evaluation at given coordinates.
   *
   * Defines new level set function. Zero level set of this level set
   * function is level set c of level set function f.
   */
  const T& operator()(const DimVec<double>& bary) const {

    f->setElInfo(elInfo_);
    return ((*f)(bary) - c);
  }; 

 protected:
  /**
   * Level set function which is translated.
   */
  ElementFunction<T> *f;

  /**
   * Translation parameter. 
   */
  double c;
};

#endif  // AMDIS_TRANSLATELSFCT_H
