#include "CoarseningManager.h"
#include "Mesh.h"
#include "AdaptStationary.h"
#include "AdaptInstationary.h"
#include "Traverse.h"
#include "MacroElement.h"
#include "RCNeighbourList.h"
#include "FixVec.h"
//#include "PeriodicBC.h"

namespace AMDiS {

  CoarseningManager* CoarseningManager::traversePtr = NULL;

  /****************************************************************************/
  /*  sets the mark on all elements that have to be coarsend                  */
  /****************************************************************************/

  int CoarseningManager::coarsenMarkFunction(ElInfo *el_info)
  {
    el_info->getElement()->setMark(traversePtr->globalMark);
    return 0;
  }

  /****************************************************************************/
  /*  tries to coarsen every element of mesh at least mark times		    */
  /****************************************************************************/

  Flag CoarseningManager::globalCoarsen(Mesh *aMesh, int mark)
  {
    if (mark >= 0) return(0);
    globalMark = mark;
    traversePtr = this;
    aMesh->traverse(-1, Mesh::CALL_LEAF_EL, coarsenMarkFunction);
    return(coarsenMesh(aMesh));
  }

  int CoarseningManager::spreadCoarsenMarkFunction(ElInfo* el_info)
  {
    Element *el = el_info->getElement();
    signed char mark;

    if (el->getChild(0)) {
      /****************************************************************************/
      /* interior node of the tree                                                */
      /****************************************************************************/
      mark = max(el->getChild(0)->getMark(), el->getChild(1)->getMark());
      el->setMark(std::min(mark + 1, 0));
    } else {
      /****************************************************************************/
      /* leaf node of the tree                                                    */
      /****************************************************************************/
      if (el->getMark() < 0) 
	el->setMark(el->getMark() - 1);
    }

    return 0;
  }

  void CoarseningManager::spreadCoarsenMark()
  {
    traversePtr = this;
    mesh->traverse(-1, Mesh::CALL_EVERY_EL_POSTORDER, spreadCoarsenMarkFunction);
  }


  /****************************************************************************/
  /*  cleanup_after_coarsen:                                   		    */
  /*  resets the element marks                                                */
  /****************************************************************************/

  int CoarseningManager::cleanUpAfterCoarsenFunction(ElInfo *el_info)
  {
    Element *el = el_info->getElement();
    el->setMark(max(el->getMark(), 0));
    return 0;
  }

  void CoarseningManager::cleanUpAfterCoarsen()
  {
    traversePtr = this;
    mesh->traverse(-1, Mesh::CALL_LEAF_EL, cleanUpAfterCoarsenFunction);
  }

  /****************************************************************************/
  /*  coarsenMesh:       							    */
  /*  traversal routine for recursiv coarsening of a triangulation            */
  /****************************************************************************/

  Flag CoarseningManager::coarsenMesh(Mesh *aMesh)
  {
    mesh = aMesh;
    int n_elements = mesh->getNumberOfLeaves();

    spreadCoarsenMark();

    stack = new TraverseStack;

    do {
      doMore = false;
      ElInfo* el_info = 
	stack->traverseFirst(mesh, -1, Mesh::CALL_EVERY_EL_POSTORDER | Mesh::FILL_NEIGH);
      while (el_info) {
	coarsenFunction(el_info);
	el_info = stack->traverseNext(el_info);
      }
    } while (doMore);

    delete stack;

    cleanUpAfterCoarsen();

    n_elements -= mesh->getNumberOfLeaves();

    return(n_elements ? MESH_COARSENED : Flag(0));
  }

}
