// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file NonLinUpdater.h */

#ifndef AMDIS_NONLINAPDATOR_H
#define AMDIS_NONLINAPDATOR_H

#include "AMDiS_fwd.h"
#include "SystemVector.h"
#include "MatrixVector.h"

namespace AMDiS {

  /** \brief
   * Used in non linear solvers for matrix and vector assemblage.
   * Base class for NonLinUpdaterScal and NonLinUpdaterVec.
   */
  template<typename Vector>
  class NonLinUpdater
  {
  public:
    virtual ~NonLinUpdater() {}

    /// Pure virtual update function.
    virtual void update(bool updateDF,
			Vector *F) = 0;
  };


  /// NonLinUpdater for scalar problems.
  class NonLinUpdaterScal : public NonLinUpdater<DOFVector<double> >
  {
  public:
    /// Constructor.
    NonLinUpdaterScal(DOFMatrix *m) : df(m) {}

    virtual ~NonLinUpdaterScal() {}

    /// Implementation of NonLinUpdater::update().
    virtual void update(bool updateDF,
			DOFVector<double> *F);

  protected:
    /// Matrix to be updated.
    DOFMatrix *df;
  };


  /// NonLinUpdater for vector valued problems.
  class NonLinUpdaterVec : public NonLinUpdater<SystemVector>
  {
  public:
    /// Constructor.
    NonLinUpdaterVec(Matrix<DOFMatrix*> *m) : df(m) {}

    virtual ~NonLinUpdaterVec() {}

    /// Implementation of NonLinUpdater::update().
    virtual void update(bool updateDF, SystemVector *F);

  protected:
    /// Matrix to be updated.
    Matrix<DOFMatrix*> *df;
  };

}

#endif
