// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file ParallelError.h */

#ifndef AMDIS_PARALLELERROR_H
#define AMDIS_PARALLELERROR_H

#include "AMDiS_fwd.h"
#include "Global.h"
#include "Mesh.h"

namespace AMDiS {

  /** \ingroup Common
   * \brief
   * True error calculator
   */
  template<typename T>
  class ParallelError
  {
  public:  
    static void setWriteLeafData() { writeInLeafData = true; }
    static void unsetWriteLeafData() { writeInLeafData = false; }
    static bool writeLeafData() { return writeInLeafData; }
  
    static double maxErrAtQp(const AbstractFunction<T, WorldVector<double> >& u,
			     const DOFVector<T>& uh,
			     const Quadrature* q);
    static double 
    H1Err(const AbstractFunction<WorldVector<T>, WorldVector<double> >& grdU,
	  const DOFVector<T>& uh,
	  //const Quadrature* quad,
	  int relErr,
	  double* max,
	  bool writeLeafData = false,
	  int comp = 0);

    static double L2Err(const AbstractFunction<T, WorldVector<double> >& u,
			const DOFVector<T>& uh,
			//Quadrature* quad,
			int relErr,
			double* max,
			bool writeLeafData = false,
			int comp = 0);

    // methods for traversal
    static int maxErrAtQpFct(ElInfo* elinfo);
    static int H1ErrFct(ElInfo* elinfo);
    static int L2ErrFct(ElInfo* elinfo);
    static int relFct(ElInfo* elinfo);
  
  public:
    static const T& errUFct(const DimVec<double>& lambda);

    static const WorldVector<T>& grdErrUFct(const DimVec<double>& lambda);

    /** \brief
     * 
     */
    class AbstrFctErrU  : public AbstractFunction<T, DimVec<double> >
    { 
    public:
      AbstrFctErrU() : AbstractFunction<T, DimVec<double> >(0) {};

      inline const T& operator()(const DimVec<double> & x) const { 
	return  ParallelError<T>::errUFct(x);
      };
    };

    static AbstrFctErrU   errU;

    /** \brief
     * 
     */
    class AbstrFctGrdErrU  : public AbstractFunction<WorldVector<T>, DimVec<double> >
    { 
    public:
      AbstrFctGrdErrU() : AbstractFunction<WorldVector<T>, DimVec<double> >(0) {};

      inline const WorldVector<T>& operator()(const DimVec<double> & x) const { 
	return   ParallelError<T> ::  grdErrUFct (x);
      };
    };

    static AbstrFctGrdErrU   grdErrU;

  private:
    static ElInfo* elinfo;
    /*   static const Parametric* el_parametric; */
    static const FastQuadrature* quadFast;
    static const AbstractFunction<T, WorldVector<double> >* pU;
    static const AbstractFunction<WorldVector<T>, WorldVector<double> >* pGrdU;
    static const BasisFunction* basFct;
    static const DOFVector<T>* errUh;
    static double maxErr;
    static double l2Err2;
    static double l2Norm2;
    static int relative;
    static double relNorm2;
    static double h1Err2;
    static double h1Norm2;
    static bool writeInLeafData;
    static int component;
  };

  template<typename T> ElInfo* ParallelError<T>::elinfo = NULL;
  /* template<typename T> const Parametric* ParallelError<T>::el_parametric = NULL; */
  template<typename T> const FastQuadrature* ParallelError<T>::quadFast = NULL;
  template<typename T> const AbstractFunction<T, WorldVector<double> >* ParallelError<T>::pU = NULL;
  template<typename T> const AbstractFunction<WorldVector<T>, WorldVector<double> >* ParallelError<T>::pGrdU = NULL;
  template<typename T> const BasisFunction* ParallelError<T>::basFct = NULL;
  template<typename T> const DOFVector<T>* ParallelError<T>::errUh = NULL;
  template<typename T> double ParallelError<T>::maxErr = 0.0;
  template<typename T> double ParallelError<T>::l2Err2 = 0.0;
  template<typename T> double ParallelError<T>::l2Norm2 = 0.0;
  template<typename T> int ParallelError<T>::relative = 0;
  template<typename T> double ParallelError<T>::relNorm2 = 0.0;
  template<typename T> double ParallelError<T>::h1Err2 = 0.0;
  template<typename T> double ParallelError<T>::h1Norm2 = 0.0;
  template<typename T> typename ParallelError<T>::AbstrFctErrU ParallelError<T>::errU;
  template<typename T> typename ParallelError<T>::AbstrFctGrdErrU ParallelError<T>::grdErrU;
  template<typename T> bool ParallelError<T>::writeInLeafData = false;
  template<typename T> int ParallelError<T>::component = 0;

}

#include "ParallelError.hh"

#endif  // AMDIS_ERROR_H



