#include "RefinementManager.h"
#include "Mesh.h"
#include "Traverse.h"
#include "ElInfo.h"
#include "DOFAdmin.h"
#include "AdaptStationary.h"
#include "AdaptInstationary.h"
#include "FixVec.h"
#include "RCNeighbourList.h"
#include "ProblemStatBase.h"
#include "PeriodicBC.h"

namespace AMDiS {

  RefinementManager* RefinementManager::traversePtr = NULL;
  bool RefinementManager::doMoreRecursiveRefine = false;
  int RefinementManager::callRefineInterpol = 0;
  RCNeighbourList* RefinementManager3d::refList = NULL;
  int RefinementManager::globalMark = 0;

  int RefinementManager::globalRefineFct(ElInfo* elinfo)
  {
    FUNCNAME("RefineManager::globalRefineFct");

    elinfo->getElement()->setMark(globalMark);
    return 0;
  }

  Flag RefinementManager::globalRefine(Mesh *aMesh, int mark)
  {
    FUNCNAME("RefinementManager::globalRefine()");

    if (mark <= 0) 
      return static_cast<Flag>(0);

    globalMark = mark;
    aMesh->traverse(-1, Mesh::CALL_LEAF_EL | Mesh::FILL_COORDS | Mesh::FILL_BOUND, 
		    globalRefineFct);
    return refineMesh(aMesh);
  }




  Flag RefinementManager::refineMesh(Mesh *aMesh)
  {
    FUNCNAME("RefinementManager::refineMesh()");

    mesh = aMesh;
    int n_elements = mesh->getNumberOfLeaves();
    ElInfo *elInfo;
    newCoords = false;
    stack = new TraverseStack;
    doMoreRecursiveRefine = true;

    while (doMoreRecursiveRefine) {
      doMoreRecursiveRefine = false;
      elInfo = stack->traverseFirst(mesh, -1, 
				    Mesh::CALL_LEAF_EL | Mesh::FILL_NEIGH | Mesh::FILL_BOUND);
      while (elInfo) {
	if (elInfo->getElement()->getMark() > 0) {
	  doMoreRecursiveRefine = 
	    doMoreRecursiveRefine || (elInfo->getElement()->getMark() > 1);
	  elInfo = refineFunction(elInfo);
	}	
	elInfo = stack->traverseNext(elInfo);
      }
    }
  
    if (newCoords) {
      setNewCoords(); // call of sub-class method  
    }

    delete stack;

    n_elements = mesh->getNumberOfLeaves() - n_elements;

    return(n_elements ? MESH_REFINED : Flag(0));
  }

}
