// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file Serializable.h */

#ifndef AMDIS_SERIALIZABLE_H
#define AMDIS_SERIALIZABLE_H

#include <iostream>
#include <string>

namespace AMDiS {

  /** \brief
   * Interface for JAVA-like serialization of objects. Serializable objects can be
   * written to an out stream and read from an in stream. So i.e. they can be stored
   * on hard disk.
   */
  class Serializable
  {
  public:
    /// Streams the object to output stream out.
    virtual void serialize(std::ostream &out) = 0;

    /// Reads the object from input stream in.
    virtual void deserialize(std::istream &in) = 0;

    /// Returns the type name for this serializable object.
    virtual std::string getTypeName() const 
    { 
      return ""; 
    }

    virtual ~Serializable() {}
  };

}

#endif
