// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file Serializer.h */

#ifndef AMDIS_SERIALIZER_H
#define AMDIS_SERIALIZER_H

#include "FileWriter.h"
#include "Parameters.h"
#include "AdaptInfo.h"
#include "ProblemStatBase.h"

namespace AMDiS {

  template<typename ProblemType>
  class Serializer : public FileWriterInterface
  {
  public:
    Serializer(ProblemType *problem) 
      : name_(""), 
	problem_(problem), 
	tsModulo_(1), 
	timestepNumber_(-1)
    {
      GET_PARAMETER(0, problem_->getName() + "->output->serialization filename", 
		    &name_);
      GET_PARAMETER(0, problem_->getName() + "->output->write every i-th timestep",
		    "%d", &tsModulo_);
      TEST_EXIT(name_ != "")("no filename\n");
    }

    virtual ~Serializer() {}

    virtual void writeFiles(AdaptInfo *adaptInfo, 
			    bool force,
			    int level = -1,
			    Flag traverseFlag = Mesh::CALL_LEAF_EL,
			    bool (*writeElem)(ElInfo*) = NULL) 
    {
      FUNCNAME("Serializer::writeFiles()");

      timestepNumber_++;
      timestepNumber_ %= tsModulo_;
      if ((timestepNumber_ != 0) && !force)
	return;

      TEST_EXIT(adaptInfo)("No AdaptInfo\n");

      std::ofstream out(name_.c_str());
      TEST_EXIT(out.is_open())("Cannot open serialization file!\n");
      problem_->serialize(out);
      adaptInfo->serialize(out);
      out.close();

      MSG("problem serialized to %s \n", name_.c_str());
    }

    void writeDelayedFiles() {}

    bool isWritingDelayed() 
    { 
      return false; 
    }

  protected:
    /// Name of file to which the problem is serialized.
    std::string name_;

    /// Pointer to the problem.
    ProblemType *problem_;

    /// The problem is serialized every tsModulo-th timestep.
    int tsModulo_;

    /// Current timestep number.
    int timestepNumber_;
  };


  class SerializerUtil {
  public:
    static void serializeInt(std::ostream &out, int* ptrInt) 
    {
      out.write(reinterpret_cast<const char*>(ptrInt), sizeof(int));
    }

    static void serializeDouble(std::ostream &out, double* ptrInt) 
    {
      out.write(reinterpret_cast<const char*>(ptrInt), sizeof(double));
    }

    static void serializeBool(std::ostream &out, bool* ptrBool) 
    {
      out.write(reinterpret_cast<const char*>(ptrBool), sizeof(bool));
    }

    static void deserializeInt(std::istream &in, int* ptrInt) 
    {
      in.read(reinterpret_cast<char*>(ptrInt), sizeof(int));
    }

    static void deserializeDouble(std::istream &in, double* ptrInt) 
    {
      in.read(reinterpret_cast<char*>(ptrInt), sizeof(double));
    }

    static void deserializeBool(std::istream &in, bool* ptrBool) 
    {
      in.read(reinterpret_cast<char*>(ptrBool), sizeof(bool));
    }

  };
}
#endif
