#include <fstream>
#include <algorithm>

#include "FixVec.h"
#include "Mesh.h"
#include "Flag.h"

namespace AMDiS {

  template<typename T> T* TecPlotWriter<T>::values = NULL;

  template<typename T> int TecPlotWriter<T>::nv = 1;

  template<typename T> std::ofstream* TecPlotWriter<T>::outFile = NULL;

  template<typename T> bool TecPlotWriter<T>::writeCoords = false;

  template<typename T> std::vector<typename TecPlotWriter<T>::DOFCoords> 
  TecPlotWriter<T>::dofCoords;

  template<typename T>
  int TecPlotWriter<T>::writeIndicesFct(ElInfo* elinfo)
  {
    const DOFAdmin* admin = values->getFESpace()->getAdmin(); 
    int n0 = admin->getNumberOfPreDOFs(VERTEX);
    const DegreeOfFreedom **dof = elinfo->getElement()->getDOF();

    int dim = elinfo->getMesh()->getDim();
    //  int dow = Global::getGeo(WORLD);

    for(int i=0; i < dim+1; i++) {
      typename DOFCoords::iterator coords = find(dofCoords[dof[i][n0]].begin(),
						 dofCoords[dof[i][n0]].end(),
						 elinfo->getCoord(i));
      TEST_EXIT(coords != dofCoords[dof[i][n0]].end())("coords not found");
      (*outFile) << coords->vertex_index << " "; 
      if(dim == 1 && i == 1) {
	(*outFile) << coords->vertex_index;     
      }
    }
    (*outFile) << std::endl;
  
    return 0;
  }

  template<typename T>
  void TecPlotWriter<T>::writeValues(T* val,
				     const char* filename,
				     const char* plotTitle,
				     bool additional)
  {
    values = val;

    TEST_EXIT(values)("no values\n");
    TEST_EXIT(filename)("no filename\n");
    TEST_EXIT(plotTitle)("no plotTitle\n");
    TEST_EXIT(values->getFESpace())("no fe-space\n");
    TEST_EXIT(values->getFESpace()->getMesh())("no mesh\n");

    if(!additional)
      outFile = new std::ofstream(filename);
    else
      outFile = new std::ofstream(filename, std::ios::app);
    
    TEST_EXIT(outFile)("can't open file %s\n", filename);

    outFile->setf(std::ios::scientific,std::ios::floatfield);

    const DOFAdmin* admin = values->getFESpace()->getAdmin(); 
    Mesh* mesh = values->getFESpace()->getMesh();

    mesh->dofCompress();
    dofCoords.resize(admin->getUsedSize());

    int dim = mesh->getDim();
    int dow = Global::getGeo(WORLD);

    writeCoords = !additional;
    
    if(dim == 1 && dow == 1) { // use ordered plot
      if(!additional) {
	(*outFile) << "TITLE = \"" << plotTitle << "\"" << std::endl;
      }

      // write file header
      if(!additional) {
	writeVarName(dim);
      }
      (*outFile) << "ZONE T=\"" << values->getName() << "\", I=" << 
	mesh->getNumberOfVertices();
      (*outFile) << ", F=POINT";
      if(additional) {
	(*outFile) << ", D=(1)";
      }
      (*outFile) << std::endl << std::endl;
    

      // write data
      mesh->traverse(-1, Mesh::CALL_LEAF_EL|Mesh::FILL_COORDS, writeValuesFct);

    } else { // dow !=1 => use FE-plot
      for(int i=0; i < 150; i++) 
	(*outFile) << " ";
      (*outFile) << std::endl;

      // write data
      nv = 1;
      mesh->traverse(-1, Mesh::CALL_LEAF_EL|Mesh::FILL_COORDS, writeValuesFct);
      (*outFile) << std::endl;
      mesh->traverse(-1, Mesh::CALL_LEAF_EL|Mesh::FILL_COORDS, writeIndicesFct);

      // write file header
      outFile->seekp(0);

      if(!additional) {
	(*outFile) << "TITLE = \"" << plotTitle << "\"" << std::endl;
	writeVarName(dim);
      }

      (*outFile) << "ZONE T=\"" << values->getName() << "\", N=" << nv-1;
      (*outFile) << ", E=" << mesh->getNumberOfLeaves();
      (*outFile) << ", F=FEPOINT";
      (*outFile) << ((dim==3) ? ", ET=TETRAHEDRON" : ", ET=TRIANGLE");
      if(additional) {
	(*outFile) << ", D=(";
	(*outFile) << ((dim==3) ? "1,2,3" : "1,2");
	(*outFile) << ",FECONNECT)";
      }
      (*outFile) << std::endl;

    }

    dofCoords.resize(0);
    delete outFile;
  }

}
