#include <cstring>
#include "ValueReader.h"
#include "MacroReader.h"

namespace AMDiS {

  /** \brief
   * Copies the values of a value file to a DOF vector.
   *
   * The DOF vector must have been created by a corresponding mesh file. The
   * information about this file and the macro triangulation are stored in
   * macroFileInfo. The function now reads the corresponding value file and
   * copies the values to the correct positions in the DOF vector.
   */
  void ValueReader::readValue(std::string filename,
			      Mesh *mesh,
			      DOFVector<double> *dofVector,
			      MacroInfo *macroFileInfo)
  {
    FUNCNAME("ValueReader::readValue()");

    TEST_EXIT(filename != "")("Filename not specified!\n");
    TEST_EXIT(mesh)("no mesh specified\n");
    TEST_EXIT(dofVector)("no DOF vector specified\n");
    TEST_EXIT(macroFileInfo)("no MacroInfo specified\n");

    std::string line;

    // open the file and read the data to the vector values.
    std::ifstream file(filename.c_str(), std::ifstream::in);

    while (!file.eof()) {
      getline(file, line);
      if (line.find("vertex values:") != std::string::npos)
	break;
    }
    // A vertex value cannot be the last line of the file, if the file is correct.
    TEST_EXIT(!file.eof())("Value file does not contain vertex values\n");

    double value;
    std::vector<double> values;
    values.clear();

    while (!file.eof()) {
      file >> value;
      if (!file.good())
	break;
      values.push_back(value);
    }

    file.close();

    // Using the macroFileInfo structure, copy the values to the
    // correct positions in the DOF vector.
    for (int i = 0; i < mesh->getNumberOfMacros(); i++) {
      for (int j = 0; j < mesh->getGeo(VERTEX); j++) {
	int fileIndex = macroFileInfo->mel_vertex[i][j];
	int dofIndex = *(macroFileInfo->dof[fileIndex]);
	(*dofVector)[dofIndex] = values[fileIndex];
      }
    }
  }
}
